#pragma hdrstop

#include "WakePort.h"
#include "Wsp32.h"

#pragma package(smart_init)

#define DEFAULT_BAUD_RATE 9600 //   
#define DEFAULT_TIMEOUT 100    //timeout  TX  RX  , 
#define Signature 0xBEDA       //   

//--------------------------- : ----------------------------------

__fastcall TWakePort::TWakePort(int maxport)
{
  CrSec = new TCriticalSection();
  FMaxPort = maxport;
  FBaudRate = DEFAULT_BAUD_RATE;
  FTxAddr = 0;     //  
  FTxCmd = C_NOP;  //  
  FTxCmdName = ""; //  
  SetNumber(0);
}

//---------------------------- : ----------------------------------

__fastcall TWakePort::~TWakePort()
{
  delete CrSec;
}

//-------------------     : ----------------------

AnsiString __fastcall TWakePort::NumberToName(int number)
{
  if((number <= FMaxPort) && (number > 0))
   return("COM"+IntToStr(number));
  return("None");
}

//-------------------     : ----------------------

int __fastcall TWakePort::NameToNumber(AnsiString name)
{
  for (int i = 0; i <= FMaxPort; i++)
   if(NumberToName(i) == name) return(i);
  return(0);
}

//-----------------------   : -----------------------------

void __fastcall TWakePort::SetName(AnsiString name)
{
  FName = name;
  FNumber = NameToNumber(FName);
}

//----------------------   : ----------------------------

void __fastcall TWakePort::SetNumber(int number)
{
  FNumber = number;
  FName = NumberToName(FNumber);
}

//---------------    ( ): -------------------

bool __fastcall TWakePort::Access(AnsiString Name)
{
  return AccessCOM(Name.c_str());
}

//---------------    ( ): ------------------

bool __fastcall TWakePort::Access(int Number)
{
  AnsiString name = NumberToName(Number);
  return(Access(name));
}

//----------------------   ( ): ---------------------------

void __fastcall TWakePort::Open(AnsiString Name, int baudrate)
{
  SetName(Name);
  FBaudRate = baudrate;
  OpenCOM(FName.c_str(), FBaudRate);
  SetModeCOM(RTS_CONTROL_TOGGLE, DTR_CONTROL_ENABLE);
}

//----------------------   ( ): --------------------------

void __fastcall TWakePort::Open(int Number, int baudrate)
{
  SetNumber(Number);
  FBaudRate = baudrate;
  Open(FName, FBaudRate);
}

//---------------------------  : ---------------------------------

void __fastcall TWakePort::Close(void)
{
  SetNumber(0);
  CloseCOM();
}

//-----------------------   : -----------------------------

void __fastcall TWakePort::Purge(void)
{
  PurgeCOM();
}

//--------------------    : ------------------------

void __fastcall TWakePort::SetTxAddr(int address)
{
  if((address < 0) || (address > 127)) FTxAddr = 0;
    else FTxAddr = address;
}

//-------------------------  : --------------------------------

void __fastcall TWakePort::TxData(int cmdcode, AnsiString cmdname,
                                  int datalength)
{
  if(FNumber) //  
  {
    FTxCmd = cmdcode;     //   
    FTxCmdName = cmdname; //   
    TxFrame((unsigned char)FTxAddr, // 
            (unsigned char)FTxCmd,
            (unsigned char)datalength,
            FTxData); //FTxData -  
  }
}

//---------------------------  : ---------------------------------

int __fastcall TWakePort::RxData(int timeout)
{
  unsigned char rxAdd = 0, rxCmd = 0, rxN = 0;
  int err;
  bool rx;

  if(FNumber) //  
  {
    rx = RxFrame((DWORD)timeout, // 
                 rxAdd,
                 rxCmd,
                 rxN,
                 FRxData); //FRxData -  

    if((rxCmd == C_INFO) || (rxCmd == C_ECHO)) err = ERR_NO;
      else err = FRxData[0];
    if(err > MAX_ERROR_CODE) err = ERR_TX;

    if(!rx)
      throw EHardwareException("  " + FTxCmdName +
      ":\r   .", 0);
      else if(rxCmd != FTxCmd)
	           throw EHardwareException("  " + FTxCmdName +
             ":\r   .", 0);
             else if(err)
	                  throw EHardwareException("  " + FTxCmdName +
                    ":\r" + Errors[err], 0);
  }
  return((int)rxN); //   
}

//---------------------------------------------------------------------------
//---------------------   : --------------------------
//---------------------------------------------------------------------------

//-------------------------  C_NOP: ---------------------------------

// C_NOP       PC
//    .

//-------------------------  C_ERR: ---------------------------------

// C_ERR     , 
//   PC,     
// .    C_ERR   ,
//    .

//-------------------------  C_ECHO: ---------------------------------

// C_ECHO     
//.      .

//-------------------------  C_INFO: ---------------------------------

// C_INFO  ,   
// ,  firmware   .
//        ,
//      .

AnsiString __fastcall TWakePort::GetInfo(void)
{
  CrSec->Enter();
  try {
    TxData(C_INFO, "GetInfo", 0);
    RxData(DEFAULT_TIMEOUT);
  } __finally { CrSec->Leave(); }
  return(AnsiString((char*)FRxData));
}

//---------------------------------------------------------------------------
//------------------   : -----------------------
//---------------------------------------------------------------------------

//  :

#define CMD_SETADDR 4 // 
#define CMD_GETADDR 5 // 
#define CMD_SETM    6 //  
#define CMD_GETM    7 //  
#define CMD_SETA    8 //     
#define CMD_GETA    9 //     
#define CMD_SETP   10 // , ,   
#define CMD_GETP   11 // , ,   
#define CMD_SETL   12 // , ,  
#define CMD_GETL   13 // , ,  
#define CMD_SETW   14 //    
#define CMD_GETW   15 //    
#define CMD_SETS   16 //  
#define CMD_GETS   17 //  
#define CMD_SETN   18 // 
#define CMD_GETN   19 // 
#define CMD_SETT   20 //  
#define CMD_GETT   21 //  

//-----------------------  CMD_SETADDR: ------------------------------

void __fastcall TWakePort::SetAddr(int a)
{
  CrSec->Enter();
  try {
    int i = 0;
    FTxData[i++] = LO(Signature);
    FTxData[i++] = HI(Signature);
    FTxData[i++] = LO(a);
    TxData(CMD_SETADDR, "SetAddr", i);
    RxData(DEFAULT_TIMEOUT);
  } __finally { CrSec->Leave(); }
}

//------------------------  CMD_GETADDR: -----------------------------

int __fastcall TWakePort::GetAddr(void)
{
  CrSec->Enter();
  int i = 0;
  try {
    TxData(CMD_GETADDR, "GetAddr", i);
    RxData(DEFAULT_TIMEOUT);
    i = FRxData[1];
  } __finally { CrSec->Leave(); }
  return(i);
}

//------------------------  CMD_SETM: --------------------------------

void __fastcall TWakePort::SetM(int vm, int ms)
{
  CrSec->Enter();
  try {
    int i = 0;
    FTxData[i++] = LO(vm);
    FTxData[i++] = HI(vm);
    FTxData[i++] = LO(ms);
    TxData(CMD_SETM, "SetM", i);
    RxData(DEFAULT_TIMEOUT);
  } __finally { CrSec->Leave(); }
}

//------------------------  CMD_GETM: --------------------------------

void __fastcall TWakePort::GetM(int &vm, int &ms)
{
  CrSec->Enter();
  try {
    int i = 0;
    TxData(CMD_GETM, "GetM", i);
    RxData(DEFAULT_TIMEOUT);
    vm = FRxData[1] + FRxData[2] * 256;
    ms = FRxData[3];
  } __finally { CrSec->Leave(); }
}

//------------------------  CMD_SETA: --------------------------------

void __fastcall TWakePort::SetA(int a, int ia)
{
  CrSec->Enter();
  try {
    int i = 0;
    FTxData[i++] = LO(a);
    FTxData[i++] = HI(a);
    FTxData[i++] = LO(ia);
    FTxData[i++] = HI(ia);
    TxData(CMD_SETA, "SetA", i);
    RxData(DEFAULT_TIMEOUT);
  } __finally { CrSec->Leave(); }
}

//------------------------  CMD_GETA: --------------------------------

void __fastcall TWakePort::GetA(int &a, int &ia)
{
  CrSec->Enter();
  try {
    int i = 0;
    TxData(CMD_GETA, "GetA", i);
    RxData(DEFAULT_TIMEOUT);
    a = FRxData[1] + FRxData[2] * 256;
    ia = FRxData[3] + FRxData[4] * 256;
  } __finally { CrSec->Leave(); }
}

//------------------------  CMD_SETP: --------------------------------

void __fastcall TWakePort::SetP(int vp, int ip, int np)
{
  CrSec->Enter();
  try {
    int i = 0;
    FTxData[i++] = LO(vp);
    FTxData[i++] = HI(vp);
    FTxData[i++] = LO(ip);
    FTxData[i++] = HI(ip);
    FTxData[i++] = LO(np);
    FTxData[i++] = HI(np);
    TxData(CMD_SETP, "SetP", i);
    RxData(DEFAULT_TIMEOUT);
  } __finally { CrSec->Leave(); }
}

//------------------------  CMD_GETP: --------------------------------

void __fastcall TWakePort::GetP(int &vp, int &ip, int &np)
{
  CrSec->Enter();
  try {
    int i = 0;
    TxData(CMD_GETP, "GetP", i);
    RxData(DEFAULT_TIMEOUT);
    vp = FRxData[1] + FRxData[2] * 256;
    ip = FRxData[3] + FRxData[4] * 256;
    np = FRxData[5] + FRxData[6] * 256;
  } __finally { CrSec->Leave(); }
}

//------------------------  CMD_SETL: --------------------------------

void __fastcall TWakePort::SetL(int vl, int il, int nl)
{
  CrSec->Enter();
  try {
    int i = 0;
    FTxData[i++] = LO(vl);
    FTxData[i++] = HI(vl);
    FTxData[i++] = LO(il);
    FTxData[i++] = HI(il);
    FTxData[i++] = LO(nl);
    FTxData[i++] = HI(nl);
    TxData(CMD_SETL, "SetL", i);
    RxData(DEFAULT_TIMEOUT);
  } __finally { CrSec->Leave(); }
}

//------------------------  CMD_GETL: --------------------------------

void __fastcall TWakePort::GetL(int &vl, int &il, int &nl)
{
  CrSec->Enter();
  try {
    int i = 0;
    TxData(CMD_GETL, "GetL", i);
    RxData(DEFAULT_TIMEOUT);
    vl = FRxData[1] + FRxData[2] * 256;
    il = FRxData[3] + FRxData[4] * 256;
    nl = FRxData[5] + FRxData[6] * 256;
  } __finally { CrSec->Leave(); }
}

//------------------------  CMD_SETW: --------------------------------

void __fastcall TWakePort::SetW(int n, int vw, int iw)
{
  CrSec->Enter();
  try {
    int i = 0;
    FTxData[i++] = LO(n);
    FTxData[i++] = LO(vw);
    FTxData[i++] = HI(vw);
    FTxData[i++] = LO(iw);
    FTxData[i++] = HI(iw);
    TxData(CMD_SETW, "SetW", i);
    RxData(DEFAULT_TIMEOUT);
  } __finally { CrSec->Leave(); }
}

//------------------------  CMD_GETW: --------------------------------

void __fastcall TWakePort::GetW(int n, int &vw, int &iw)
{
  CrSec->Enter();
  try {
    int i = 0;
    FTxData[i++] = LO(n);
    TxData(CMD_GETW, "GetW", i);
    RxData(DEFAULT_TIMEOUT);
    vw = FRxData[1] + FRxData[2] * 256;
    iw = FRxData[3] + FRxData[4] * 256;
  } __finally { CrSec->Leave(); }
}

//------------------------  CMD_SETS: --------------------------------

void __fastcall TWakePort::SetS(int s)
{
  CrSec->Enter();
  try {
    int i = 0;
    FTxData[i++] = LO(s);
    TxData(CMD_SETS, "SetS", i);
    RxData(DEFAULT_TIMEOUT);
  } __finally { CrSec->Leave(); }
}

//------------------------  CMD_GETS: --------------------------------

void __fastcall TWakePort::GetS(int &state, int &sw, int &dip, int &pc)
{
  CrSec->Enter();
  try {
    int i = 0;
    TxData(CMD_GETS, "GetS", i);
    RxData(DEFAULT_TIMEOUT);
    state = FRxData[1];
    sw = FRxData[2];
    dip = FRxData[3];
    pc = FRxData[4];
  } __finally { CrSec->Leave(); }
}

//-------------------------  CMD_SETN: -------------------------------

void __fastcall TWakePort::SetN(int n)
{
  CrSec->Enter();
  try {
    int i = 0;
    FTxData[i++] = LO(n);
    FTxData[i++] = HI(n);
    TxData(CMD_SETN, "SetN", i);
    RxData(DEFAULT_TIMEOUT);
  } __finally { CrSec->Leave(); }
}

//--------------------------  CMD_GETN: ------------------------------

int __fastcall TWakePort::GetN(void)
{
  CrSec->Enter();
  int i = 0; short s;
  try {
    TxData(CMD_GETN, "GetN", i);
    RxData(DEFAULT_TIMEOUT);
    s = FRxData[1] + FRxData[2] * 256;
  } __finally { CrSec->Leave(); }
  return(s);
}

//-------------------------  CMD_SETT: -------------------------------

void __fastcall TWakePort::SetT(int n, int l)
{
  CrSec->Enter();
  try {
    int i = 0;
    FTxData[i++] = LO(n);
    FTxData[i++] = HI(n);
    FTxData[i++] = LO(l);
    FTxData[i++] = HI(l);
    TxData(CMD_SETT, "SetT", i);
    RxData(DEFAULT_TIMEOUT);
  } __finally { CrSec->Leave(); }
}

//--------------------------  CMD_GETT: ------------------------------

void __fastcall TWakePort::GetT(int &n, int &l)
{
  CrSec->Enter();
  int i = 0; short s;
  try {
    TxData(CMD_GETT, "GetT", i);
    RxData(DEFAULT_TIMEOUT);
    n = FRxData[1] + FRxData[2] * 256;
    l = FRxData[3] + FRxData[4] * 256;
  } __finally { CrSec->Leave(); }
}

//---------------------------------------------------------------------------


