#ifndef WakePortH
#define WakePortH

#include <SysUtils.hpp>
#include <SyncObjs.hpp>

#define DEVICE_TYPEN "  "
#define DEVICE_NAME "-1900"
#define E_MAIL "wubblick@yahoo.com"

enum {C_NOP, C_ERR, C_ECHO, C_INFO}; //   Wake
enum {ERR_NO, ERR_TX, ERR_BU, ERR_RE, ERR_PA, ERR_NR, ERR_NC};
#define MAX_ERROR_CODE ERR_NC
const AnsiString Errors[] = {
   "",
   " .",
   " .",
   "  .",
   " .",
   "  ."
   " ."
};

#define HI(x) ((x >> 8) & 0xFF)
#define LO(x) (x & 0xFF)

// :

enum { ST_STOP,          // 
       ST_OPEN,          // 
       ST_CLOSE,         // 
       ST_PLAY_OPEN,     //   
       ST_PLAY_CLOSE,    //   
       ST_LOCK_OPEN,     //  
       ST_LOCK_CLOSE,    //  
       ST_LOCKED_OPEN,   //  
       ST_LOCKED_CLOSE,  //  
       ST_UNLOCK_OPEN,   //  
       ST_UNLOCK_CLOSE,  //  
       ST_HALT,          //   
       ST_POWER_OFF,     //    
       ST_BLCKED,        // 
       ST_HCM,           //  
       ST_FAIL };        // 

// sw:

// bool Sw_Opn : 1; // 
// bool Sw_Cls : 1; // 
// bool Sw_LmO : 1; //    
// bool Sw_LmC : 1; //    
// bool Sw_BlO : 1; //   
// bool Sw_BlC : 1; //   
// bool Sw_HCM : 1; //  
// bool Sw_ERR : 1; //   

// dip:

// char Speed : 2;  //  
// bool LockO : 1;  //     
// bool LockC : 1;  //     

#define STP 0x00    // 
#define OPN 0x01    // 
#define CLS 0x02    // 
#define PCE 0x04    //    

//--------------------   : -----------------------

class TWakePort : public TObject
{
private:
  int FMaxPort;        //  
  AnsiString FName;    //  
  int FNumber;         // 
  int FBaudRate;       // 

  AnsiString __fastcall NumberToName(int number);
  int __fastcall NameToNumber(AnsiString name);
  void __fastcall SetNumber(int number);    //  
  void __fastcall SetName(AnsiString name); //  

  int FTxAddr;           //  
  int FTxCmd;            // 
  AnsiString FTxCmdName; //  

  int FRxAddr;           //  
  int FCmdCode;          // 
  AnsiString FCmdName;   // 

  TCriticalSection *CrSec; // 

protected:

  unsigned char FTxData[256]; //     
  unsigned char FRxData[256]; //     

public:
  __fastcall TWakePort(int maxport); //
  __fastcall ~TWakePort();           //

  bool __fastcall Access(int Number); //    
  bool __fastcall Access(AnsiString Name); //    
  void __fastcall Open(int Number, int baudrate); //   
  void __fastcall Open(AnsiString Name, int baudrate); //   
  void __fastcall Close(); // 
  void __fastcall Purge(void); //  
  void __fastcall SetTxAddr(int address); //   
  void __fastcall TxData(int cmdcode, AnsiString cmdname, int datalength);
  int __fastcall RxData(int timeout);

  __property int Number = {read=FNumber, write=SetNumber};
  __property AnsiString Name = {read=FName, write=SetName};

  AnsiString __fastcall GetInfo(void);
  void __fastcall SetAddr(int a);
  int __fastcall GetAddr(void);
  void __fastcall SetM(int vm, int ms);
  void __fastcall GetM(int &vm, int &ms);
  void __fastcall SetA(int a, int ia);
  void __fastcall GetA(int &a, int &ia);
  void __fastcall SetP(int vp, int ip, int np);
  void __fastcall GetP(int &vp, int &ip, int &np);
  void __fastcall SetL(int vl, int il, int nl);
  void __fastcall GetL(int &vl, int &il, int &nl);
  void __fastcall SetW(int n, int vw, int iw);
  void __fastcall GetW(int n, int &vw, int &iw);
  void __fastcall SetS(int s);
  void __fastcall GetS(int &state, int &sw, int &dip, int &pc);
  void __fastcall SetN(int n);
  int __fastcall GetN(void);
  void __fastcall SetT(int n, int l);
  void __fastcall GetT(int &n, int &l);
};

//---------------------     : ------------------------

class EHardwareException : public Exception {
public:
  __fastcall EHardwareException(const AnsiString Msg) : Exception(Msg) {};
  __fastcall EHardwareException(const AnsiString Msg, int AHelpContext) :
	Exception(Msg, AHelpContext) {};
};

//---------------------------------------------------------------------------

#endif
